//-----------------------------------------------------------------------
//	File:		$RCSfile: Lab1Circuit.v,v $
//	Version:	$Revision: 1.1 $
//	Desc:		Lab1Circuit
//	Author:		Greg Gibeling
//	Copyright:	Copyright 2003-2005 UC Berkeley
//	This copyright header must appear in all derivative works.
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
//	Section:	Change Log
//-----------------------------------------------------------------------
//	$Log: Lab1Circuit.v,v $
//	Revision 1.1  2005/01/17 22:47:39  gdgib
//	Finished modifications from Fall 2004
//	Fully Tested
//	
//	Revision 1.3  2004/08/30 03:42:46  Administrator
//	Fixed carry bug
//	
//	Revision 1.2  2004/07/09 23:41:15  Administrator
//	Added comments
//	Used better instance names
//	
//	Revision 1.1  2004/07/02 03:05:14  Administrator
//	Initial Import
//	Not tested
//	
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
//	Module:		Lab1Circuit
//	Desc:		This is a mystery circuit, it is your job to try
//			and understand it, to see what it does...
//			We recommend simulation!
//-----------------------------------------------------------------------
module Lab1Circuit(In, Out, Enable, Clock, Reset);
	//---------------------------------------------------------------
	//	I/O
	//---------------------------------------------------------------
	input	[7:0]		In;
	output	[7:0]		Out;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Control Signals
	//---------------------------------------------------------------
	input			Enable;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	System Inputs
	//---------------------------------------------------------------
	input			Clock, Reset;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Internal Cascade Bus
	//---------------------------------------------------------------
	wire	[6:0]		CBus;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Individual Bit Circuits
	//---------------------------------------------------------------
	Lab1Cell	Bit0(In[0], Out[0], 1'b0,    CBus[0], Enable, Clock, Reset);
	Lab1Cell	Bit1(In[1], Out[1], CBus[0], CBus[1], Enable, Clock, Reset);
	Lab1Cell	Bit2(In[2], Out[2], CBus[1], CBus[2], Enable, Clock, Reset);
	Lab1Cell	Bit3(In[3], Out[3], CBus[2], CBus[3], Enable, Clock, Reset);
	Lab1Cell	Bit4(In[4], Out[4], CBus[3], CBus[4], Enable, Clock, Reset);
	Lab1Cell	Bit5(In[5], Out[5], CBus[4], CBus[5], Enable, Clock, Reset);
	Lab1Cell	Bit6(In[6], Out[6], CBus[5], CBus[6], Enable, Clock, Reset);
	Lab1Cell	Bit7(In[7], Out[7], CBus[6],        , Enable, Clock, Reset);
	//---------------------------------------------------------------
endmodule
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
//	Module:		Lab1Cell
//	Desc:		This is a single part of the Lab1Circuit above.
//			Understanding this is key to understanding the
//			Lab1Circuit.
//-----------------------------------------------------------------------
module Lab1Cell(In, Out, CIn, COut, Enable, Clock, Reset);
	//---------------------------------------------------------------
	//	I/O
	//---------------------------------------------------------------
	input			In, CIn;
	output			Out, COut;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Control Signals
	//---------------------------------------------------------------
	input			Enable;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	System Inputs
	//---------------------------------------------------------------
	input			Clock, Reset;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Wires
	//---------------------------------------------------------------
	wire			SIntermediate, SFinal, CPropagrate, CGenerate;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Combinational Logic
	//---------------------------------------------------------------
	xor	xor1(	SIntermediate,	In,		Out);
	and	and1(	CGenerate,	In,		Out);
	xor	xor2(	SFinal,		SIntermediate,	CIn);
	and	and2(	CPropagate,	SIntermediate,	CIn);
	or	or1(	COut,		CGenerate,	CPropagate);
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Sequential Logic
	//---------------------------------------------------------------
	FDCE	FF(	.Q(		Out),
			.C(		Clock),
			.CE(		Enable),
			.CLR(		Reset),
			.D(		SFinal));
	//---------------------------------------------------------------
endmodule
//-----------------------------------------------------------------------